From a673f26b65956541ac06d4f649993e7eb8f70db2 Mon Sep 17 00:00:00 2001
From: Mathieu Lemoine <lemoinem@iro.umontreal.ca>
Date: Tue, 18 May 2010 16:20:26 -0400
Subject: [PATCH] add lastcons


diff --git a/lists.lisp b/lists.lisp
index ffe418a..55930d4 100644
--- a/lists.lisp
+++ b/lists.lisp
@@ -229,6 +229,23 @@ list."
     (setf (cadr last) object)
     (setf (car fast) object)))
 
+(defun lastcons (list)
+  "Returns the last cons of LIST. Signals a type-error if LIST is not a
+proper list."
+  (if (endp (cdr list))
+      list
+      (do ((last list fast)
+           (fast list (cddr fast))
+           (slow (cons (car list) (cdr list)) (cdr slow)))
+          (nil)
+        (declare (dynamic-extent slow))
+        (when (safe-endp fast)
+          (return (lastcons slow)))
+        (when (safe-endp (cdr fast))
+          (return fast))
+        (when (eq fast slow)
+          (circular-list-error list)))))
+
 (defun make-circular-list (length &key initial-element)
   "Creates a circular list of LENGTH with the given INITIAL-ELEMENT."
   (let ((cycle (make-list length :initial-element initial-element)))
